@@ -14,7 +14,7 @@ from contextlib import nested, closing
 
 
 from calibre import (browser, __appname__, iswindows, force_unicode,
-                    strftime, preferred_encoding, as_unicode)
+                    strftime, preferred_encoding, as_unicode, random_user_agent)
 from calibre.ebooks.BeautifulSoup import BeautifulSoup, NavigableString, CData, Tag
 from calibre.ebooks.metadata.opf2 import OPFCreator
 from calibre import entity_to_unicode
@@ -49,7 +49,7 @@ class BasicNewsRecipe(Recipe):
     to creating recipes, see :doc:`news`.
     '''
 
-    #: The title to use for the ebook
+    #: The title to use for the e-book
     title                  = _('Unknown News Source')
 
     #: A couple of lines that describe the content this recipe downloads.
@@ -106,7 +106,7 @@ class BasicNewsRecipe(Recipe):
 
     #: Convenient flag to disable loading of stylesheets for websites
     #: that have overly complex stylesheets unsuitable for conversion
-    #: to ebooks formats.
+    #: to e-book formats.
     #: If True stylesheets are not downloaded and processed
     no_stylesheets         = False
 
@@ -226,7 +226,7 @@ class BasicNewsRecipe(Recipe):
     #:    }
     #:
     #: All keys are optional. For a full explanation of the search criteria, see
-    #: `Beautiful Soup <http://www.crummy.com/software/BeautifulSoup/bs3/documentation.html#Searching%20the%20Parse%20Tree>`_
+    #: `Beautiful Soup <https://www.crummy.com/software/BeautifulSoup/bs3/documentation.html#Searching%20the%20Parse%20Tree>`_
     #: A common example::
     #:
     #:   remove_tags = [dict(name='div', attrs={'class':'advert'})]
@@ -320,8 +320,8 @@ class BasicNewsRecipe(Recipe):
     #: are expressed as a percentage of the downloaded cover.
     #: cover_margins = (10, 15, '#ffffff') pads the cover with a white margin
     #: 10px on the left and right, 15px on the top and bottom.
-    #: Color names defined at http://www.imagemagick.org/script/color.php
-    #: Note that for some reason, white does not always work on windows. Use
+    #: Color names defined at https://www.imagemagick.org/script/color.php
+    #: Note that for some reason, white does not always work in Windows. Use
     #: #ffffff instead
     cover_margins = (0, 0, '#ffffff')
 
@@ -389,6 +389,10 @@ class BasicNewsRecipe(Recipe):
     #: with the URL scheme of your particular website.
     resolve_internal_links = False
 
+    #: Set to True if you want to use gziped transfers. Note that some old servers flake out with this
+    #: so it is off by default.
+    handle_gzip = False
+
     # See the built-in recipes for examples of these settings.
 
     def short_title(self):
@@ -401,7 +405,7 @@ class BasicNewsRecipe(Recipe):
         ignore it.
 
         :param url: The URL to be followed
-        :param tag: The Tag from which the URL was derived
+        :param tag: The tag from which the URL was derived
         '''
         raise NotImplementedError
 
@@ -468,10 +472,18 @@ class BasicNewsRecipe(Recipe):
         '''
         return url
 
+    def preprocess_image(self, img_data, image_url):
+        '''
+        Perform some processing on downloaded image data. This is called on the raw
+        data before any resizing is done. Must return the processed raw data. Return
+        None to skip the image.
+        '''
+        return img_data
+
     def get_browser(self, *args, **kwargs):
         '''
         Return a browser instance used to fetch documents from the web. By default
-        it returns a `mechanize <http://wwwsearch.sourceforge.net/mechanize/>`_
+        it returns a `mechanize <https://github.com/jjlee/mechanize>`_
         browser instance that supports cookies, ignores robots.txt, handles
         refreshes and has a mozilla firefox user agent.
 
@@ -482,7 +494,7 @@ class BasicNewsRecipe(Recipe):
             def get_browser(self):
                 br = BasicNewsRecipe.get_browser(self)
                 if self.username is not None and self.password is not None:
-                    br.open('http://www.nytimes.com/auth/login')
+                    br.open('https://www.nytimes.com/auth/login')
                     br.select_form(name='login')
                     br['USERID']   = self.username
                     br['PASSWORD'] = self.password
@@ -490,8 +502,13 @@ class BasicNewsRecipe(Recipe):
                 return br
 
         '''
+        if 'user_agent' not in kwargs:
+            # More and more news sites are serving JPEG XR images to IE
+            kwargs['user_agent'] = random_user_agent(allow_ie=False)
         br = browser(*args, **kwargs)
         br.addheaders += [('Accept', '*/*')]
+        if self.handle_gzip:
+            br.set_handle_gzip(True)
         return br
 
     def clone_browser(self, br):
@@ -535,7 +552,7 @@ class BasicNewsRecipe(Recipe):
         for key in article.keys():
             if key.endswith('_origlink'):
                 url = article[key]
-                if url and url.startswith('http://'):
+                if url and (url.startswith('http://') or url.startswith('https://')):
                     return url
         ans = article.get('link', None)
         if not ans and getattr(article, 'links', None):
@@ -554,7 +571,7 @@ class BasicNewsRecipe(Recipe):
         an ad page, return the HTML of the real page. Otherwise return
         None.
 
-        `soup`: A `BeautifulSoup <http://www.crummy.com/software/BeautifulSoup/bs3/documentation.html>`_
+        `soup`: A `BeautifulSoup <https://www.crummy.com/software/BeautifulSoup/bs3/documentation.html>`_
         instance containing the downloaded :term:`HTML`.
         '''
         return None
@@ -596,7 +613,7 @@ class BasicNewsRecipe(Recipe):
         It can be used to do arbitrarily powerful pre-processing on the :term:`HTML`.
         It should return `soup` after processing it.
 
-        `soup`: A `BeautifulSoup <http://www.crummy.com/software/BeautifulSoup/bs3/documentation.html>`_
+        `soup`: A `BeautifulSoup <https://www.crummy.com/software/BeautifulSoup/bs3/documentation.html>`_
         instance containing the downloaded :term:`HTML`.
         '''
         return soup
@@ -608,7 +625,7 @@ class BasicNewsRecipe(Recipe):
         It can be used to do arbitrarily powerful post-processing on the :term:`HTML`.
         It should return `soup` after processing it.
 
-        :param soup: A `BeautifulSoup <http://www.crummy.com/software/BeautifulSoup/bs3/documentation.html>`_  instance containing the downloaded :term:`HTML`.
+        :param soup: A `BeautifulSoup <https://www.crummy.com/software/BeautifulSoup/bs3/documentation.html>`_  instance containing the downloaded :term:`HTML`.
         :param first_fetch: True if this is the first page of an article.
 
         '''
@@ -643,7 +660,7 @@ class BasicNewsRecipe(Recipe):
     def index_to_soup(self, url_or_raw, raw=False, as_tree=False):
         '''
         Convenience method that takes an URL to the index page and returns
-        a `BeautifulSoup <http://www.crummy.com/software/BeautifulSoup/bs3/documentation.html>`_
+        a `BeautifulSoup <https://www.crummy.com/software/BeautifulSoup/bs3/documentation.html>`_
         of it.
 
         `url_or_raw`: Either a URL or the downloaded index page as a string
@@ -823,9 +840,9 @@ class BasicNewsRecipe(Recipe):
         Called when each HTML page belonging to article is downloaded.
         Intended to be used to get article metadata like author/summary/etc.
         from the parsed HTML (soup).
+
         :param article: A object of class :class:`calibre.web.feeds.Article`.
-        If you change the summary, remember to also change the
-        text_summary
+            If you change the summary, remember to also change the text_summary
         :param soup: Parsed HTML belonging to this article
         :param first: True iff the parsed HTML is the first page of the article.
         '''
@@ -920,6 +937,7 @@ class BasicNewsRecipe(Recipe):
             setattr(self.web2disk_options, extra, getattr(self, extra))
 
         self.web2disk_options.postprocess_html = self._postprocess_html
+        self.web2disk_options.preprocess_image = self.preprocess_image
         self.web2disk_options.encoding = self.encoding
         self.web2disk_options.preprocess_raw_html = self.preprocess_raw_html_
 
@@ -1239,7 +1257,9 @@ class BasicNewsRecipe(Recipe):
         for req in self.jobs:
             tp.putRequest(req, block=True, timeout=0)
 
-        self.report_progress(0, _('Starting download [%d thread(s)]...')%self.simultaneous_downloads)
+        self.report_progress(0, ngettext(
+            'Starting download in a single thread...',
+            'Starting download [{} threads]...', self.simultaneous_downloads).format(self.simultaneous_downloads))
         while True:
             try:
                 tp.poll()
@@ -1489,7 +1509,7 @@ class BasicNewsRecipe(Recipe):
                     for curl in self.canonicalize_internal_url(a.orig_url, is_link=False):
                         aumap[curl].add(arelpath)
                     parent.add_item(arelpath, None,
-                            a.title if a.title else _('Untitled Article'),
+                            a.title if a.title else _('Untitled article'),
                             play_order=po, author=auth,
                             description=desc, toc_thumbnail=tt)
                     last = os.path.join(self.output_dir, ('%sindex.html'%adir).replace('/', os.sep))
@@ -1625,14 +1645,14 @@ class BasicNewsRecipe(Recipe):
     def tag_to_string(self, tag, use_alt=True, normalize_whitespace=True):
         '''
         Convenience method to take a
-        `BeautifulSoup <http://www.crummy.com/software/BeautifulSoup/bs3/documentation.html>`_
+        `BeautifulSoup <https://www.crummy.com/software/BeautifulSoup/bs3/documentation.html>`_
         `Tag` and extract the text from it recursively, including any CDATA sections
         and alt tag attributes. Return a possibly empty unicode string.
 
         `use_alt`: If `True` try to use the alt attribute for tags that don't
         have any textual content
 
-        `tag`: `BeautifulSoup <http://www.crummy.com/software/BeautifulSoup/bs3/documentation.html>`_
+        `tag`: `BeautifulSoup <https://www.crummy.com/software/BeautifulSoup/bs3/documentation.html>`_
         `Tag`
         '''
         if tag is None:
@@ -1753,7 +1773,7 @@ class CalibrePeriodical(BasicNewsRecipe):
     #: Set this to the slug for the calibre periodical
     calibre_periodicals_slug = None
 
-    LOG_IN = 'http://news.calibre-ebook.com/accounts/login'
+    LOG_IN = 'https://news.calibre-ebook.com/accounts/login'
     needs_subscription = True
     __author__ = 'calibre Periodicals'
 
@@ -1775,7 +1795,7 @@ class CalibrePeriodical(BasicNewsRecipe):
         self.log('Fetching downloaded recipe')
         try:
             raw = self.browser.open_novisit(
-                'http://news.calibre-ebook.com/subscribed_files/%s/0/temp.downloaded_recipe'
+                'https://news.calibre-ebook.com/subscribed_files/%s/0/temp.downloaded_recipe'
                 % self.calibre_periodicals_slug
                     ).read()
         except Exception as e:
@@ -1799,4 +1819,3 @@ class CalibrePeriodical(BasicNewsRecipe):
         except:
             self.log.exception('Failed to compile downloaded recipe')
         return os.path.abspath('index.html')
-
